@@ -64,6 +64,8 @@ struct ksmbd_conn *ksmbd_conn_alloc(void)
 		conn->local_nls = load_nls_default();
 	atomic_set(&conn->req_running, 0);
 	atomic_set(&conn->r_count, 0);
+	conn->total_credits = 1;
+
 	init_waitqueue_head(&conn->req_running_q);
 	INIT_LIST_HEAD(&conn->conns_list);
 	INIT_LIST_HEAD(&conn->sessions);
@@ -186,7 +188,7 @@ int ksmbd_conn_write(struct ksmbd_work *work)
 
 	if (work->tr_buf) {
 		iov[iov_idx] = (struct kvec) { work->tr_buf,
-				sizeof(struct smb2_transform_hdr) + 4 };
+				sizeof(struct smb2_transform_hdr) };
 		len += iov[iov_idx++].iov_len;
 	}
 
@@ -319,6 +321,9 @@ int ksmbd_conn_handler_loop(void *p)
 			continue;
 		}
 
+		if (pdu_size > MAX_STREAM_PROT_LEN)
+                        continue;
+
 		/* 4 for rfc1002 length field */
 		size = pdu_size + 4;
 		conn->request_buf = kvmalloc(size, GFP_KERNEL);
