@@ -8,12 +8,11 @@
 #define __KSMBD_GLOB_H
 
 #include <linux/ctype.h>
-#include <linux/version.h>
 
 #include "unicode.h"
 #include "vfs_cache.h"
 
-#define KSMBD_VERSION	"3.4.2"
+#define KSMBD_VERSION	"3.4.3"
 
 extern int ksmbd_debug_types;
 
@@ -51,4 +50,9 @@ extern int ksmbd_debug_types;
 /* ksmbd misc functions */
 extern void ntstatus_to_dos(__le32 ntstatus, __u8 *eclass, __le16 *ecode);
 #endif
+
+#ifndef LOOKUP_NO_SYMLINKS
+#define LOOKUP_NO_SYMLINKS 0
+#endif
+
 #endif /* __KSMBD_GLOB_H */
